@@ -22,7 +22,9 @@
 #endif
 
 #include "gcal-application.h"
+#include "css-code.h"
 #include "gcal-window.h"
+#include "gcal-shell-search-provider.h"
 
 #include <glib.h>
 #include <glib-object.h>
@@ -30,27 +32,22 @@
 #include <glib/gi18n.h>
 
 #define CSS_FILE "resource:///org/gnome/calendar/gtk-styles.css"
-#define CSS_TEMPLATE "GcalEventWidget.color-%d { background-color: %s; }"
 
-typedef struct
+struct _GcalApplicationPrivate
 {
   GtkWidget      *window;
 
   GSettings      *settings;
   GcalManager    *manager;
 
-  GtkCssProvider *provider;
+  GcalShellSearchProvider *search_provider;
 
-  gchar         **css_code_snippets;
+  GtkCssProvider *provider;
   GtkCssProvider *colors_provider;
 
+  gchar          *uuid;
   icaltimetype   *initial_date;
-} GcalApplicationPrivate;
-
-static void     source_added_cb                       (GcalManager         *manager,
-                                                       ESource             *source,
-                                                       gboolean             enabled,
-                                                       gpointer             user_data);
+};
 
 static void     gcal_application_finalize             (GObject                 *object);
 
@@ -83,10 +80,20 @@ static void     gcal_application_quit                 (GSimpleAction           *
                                                        GVariant                *parameter,
                                                        gpointer                 user_data);
 
+static gboolean gcal_application_dbus_register       (GApplication             *application,
+                                                      GDBusConnection          *connection,
+                                                      const gchar              *object_path,
+                                                      GError                  **error);
+
+static void     gcal_application_dbus_unregister     (GApplication             *application,
+                                                      GDBusConnection          *connection,
+                                                      const gchar              *object_path);
+
 G_DEFINE_TYPE_WITH_PRIVATE (GcalApplication, gcal_application, GTK_TYPE_APPLICATION);
 
 static gboolean show_version = FALSE;
 static gchar* date = NULL;
+static gchar* uuid = NULL;
 
 static GOptionEntry gcal_application_goptions[] = {
   {
@@ -99,6 +106,11 @@ static GOptionEntry gcal_application_goptions[] = {
     G_OPTION_ARG_STRING, &date,
     N_("Open calendar on the passed date"), NULL
   },
+  {
+    "uuid", 'u', 0,
+    G_OPTION_ARG_STRING, &uuid,
+    N_("Open calendar showing the passed event"), NULL
+  },
   { NULL }
 };
 
@@ -111,41 +123,42 @@ static const GActionEntry gcal_app_entries[] = {
 };
 
 static void
-source_added_cb (GcalManager *manager,
-                 ESource     *source,
-                 gboolean     enabled,
-                 gpointer     user_data)
+process_sources (GcalApplication *application)
 {
   GcalApplicationPrivate *priv;
 
-  gint i, arr_length = 0;
-  ESourceSelectable *extension;
+  GList *sources, *l;
+  ESource *source;
+
+  gint arr_length, i = 0;
   GQuark color_id;
+  const gchar* color_str;
 
   gchar **new_css_snippets;
   gchar *new_css_data;
 
   GError *error = NULL;
 
-  priv = gcal_application_get_instance_private (GCAL_APPLICATION (user_data));
-
-  extension = E_SOURCE_SELECTABLE (e_source_get_extension (source, E_SOURCE_EXTENSION_CALENDAR));
-  color_id = g_quark_from_string (e_source_selectable_get_color (extension));
-
-  if (priv->css_code_snippets != NULL)
-    arr_length = g_strv_length (priv->css_code_snippets);
+  priv = application->priv;
 
+  sources = gcal_manager_get_sources_connected (priv->manager);
+  arr_length = g_list_length (sources);
   new_css_snippets = g_new0 (gchar*, arr_length + 2);
+  for (l = sources; l != NULL; l = g_list_next (l), i++)
+    {
+      source = l->data;
 
-  for (i = 0; i < arr_length; i++)
-    new_css_snippets[i] = priv->css_code_snippets[i];
-  new_css_snippets[arr_length] = g_strdup_printf (CSS_TEMPLATE, color_id, e_source_selectable_get_color (extension));
+      color_str = get_color_name_from_source (source);
+      color_id = g_quark_from_string (color_str);
 
-  if (priv->css_code_snippets != NULL)
-    g_free (priv->css_code_snippets); /* shallow free, cause I'm just moving the pointers */
-  priv->css_code_snippets = new_css_snippets;
+      new_css_snippets[i] = g_strdup_printf (CSS_TEMPLATE, color_id, color_str);
+    }
+
+  g_list_free (sources);
 
   new_css_data = g_strjoinv ("\n", new_css_snippets);
+  g_strfreev (new_css_snippets);
+
   error = NULL;
   gtk_css_provider_load_from_data (priv->colors_provider, new_css_data, -1, &error);
   if (error != NULL)
@@ -154,6 +167,15 @@ source_added_cb (GcalManager *manager,
   g_free (new_css_data);
 }
 
+static void
+sources_added_cb (GcalApplication *application,
+                  ESource         *source,
+                  gboolean         enabled,
+                  GcalManager     *manager)
+{
+  process_sources (application);
+}
+
 static void
 gcal_application_class_init (GcalApplicationClass *klass)
 {
@@ -167,85 +189,62 @@ gcal_application_class_init (GcalApplicationClass *klass)
   application_class->activate = gcal_application_activate;
   application_class->startup = gcal_application_startup;
   application_class->command_line = gcal_application_command_line;
+
+  application_class->dbus_register = gcal_application_dbus_register;
+  application_class->dbus_unregister = gcal_application_dbus_unregister;
 }
 
 static void
 gcal_application_init (GcalApplication *self)
 {
-  GcalApplicationPrivate *priv;
+  GcalApplicationPrivate *priv = gcal_application_get_instance_private (self);
 
-  priv = gcal_application_get_instance_private (self);
   priv->settings = g_settings_new ("org.gnome.calendar");
   priv->colors_provider = gtk_css_provider_new ();
+
+  priv->manager = gcal_manager_new_with_settings (priv->settings);
+  g_signal_connect_swapped (priv->manager, "source-added", G_CALLBACK (sources_added_cb), self);
+
+  priv->search_provider = gcal_shell_search_provider_new ();
+  gcal_shell_search_provider_connect (priv->search_provider, priv->manager);
+
+  self->priv = priv;
 }
 
 static void
 gcal_application_finalize (GObject *object)
 {
- GcalApplicationPrivate *priv;
-
-  priv = gcal_application_get_instance_private (GCAL_APPLICATION (object));
-
-  gtk_style_context_remove_provider_for_screen (gdk_screen_get_default (), GTK_STYLE_PROVIDER (priv->colors_provider));
-  gtk_style_context_remove_provider_for_screen (gdk_screen_get_default (), GTK_STYLE_PROVIDER (priv->provider));
-
-  g_clear_object (&(priv->settings));
-  g_clear_object (&(priv->colors_provider));
-  g_clear_object (&(priv->provider));
-  g_clear_object (&(priv->manager));
+ GcalApplicationPrivate *priv = GCAL_APPLICATION (object)->priv;
 
+  g_free (priv->uuid);
   if (priv->initial_date != NULL)
     g_free (priv->initial_date);
 
-  if (priv->css_code_snippets != NULL)
-    g_strfreev (priv->css_code_snippets);
+  if (priv->provider != NULL)
+    {
+      gtk_style_context_remove_provider_for_screen (gdk_screen_get_default (), GTK_STYLE_PROVIDER (priv->colors_provider));
+
+      gtk_style_context_remove_provider_for_screen (gdk_screen_get_default (), GTK_STYLE_PROVIDER (priv->provider));
+      g_clear_object (&(priv->provider));
+    }
+
+  g_clear_object (&(priv->colors_provider));
+  g_clear_object (&(priv->settings));
+
+  g_clear_object (&(priv->manager));
 
   G_OBJECT_CLASS (gcal_application_parent_class)->finalize (object);
+  g_clear_object (&(priv->search_provider));
 }
 
 static void
 gcal_application_activate (GApplication *application)
-{
-  GcalApplicationPrivate *priv;
-
-  priv = gcal_application_get_instance_private (GCAL_APPLICATION (application));
-
-  if (priv->window != NULL)
-    {
-      gtk_window_present (GTK_WINDOW (priv->window));
-    }
-  else
-    {
-      if (priv->initial_date == NULL)
-        {
-          priv->initial_date = g_new0 (icaltimetype, 1);
-          /* FIXME: here read the initial date from somewehere */
-          *(priv->initial_date) = icaltime_current_time_with_zone (gcal_manager_get_system_timezone (priv->manager));
-          *(priv->initial_date) = icaltime_set_timezone (priv->initial_date,
-                                                         gcal_manager_get_system_timezone (priv->manager));
-        }
-
-      priv->window = gcal_window_new_with_view_and_date (GCAL_APPLICATION (application),
-                                                         g_settings_get_enum (priv->settings, "active-view"),
-                                                         priv->initial_date);
-      g_settings_bind (priv->settings, "active-view", priv->window, "active-view",
-                       G_SETTINGS_BIND_SET | G_SETTINGS_BIND_GET);
-
-      /* FIXME: remove me in favor of gtk_widget_show() */
-      gtk_widget_show_all (priv->window);
-    }
-}
-
-static void
-gcal_application_startup (GApplication *app)
 {
   GcalApplicationPrivate *priv;
   GFile* css_file;
   GError *error;
 
-  priv = gcal_application_get_instance_private (GCAL_APPLICATION (app));
-
-  G_APPLICATION_CLASS (gcal_application_parent_class)->startup (app);
+  priv = GCAL_APPLICATION (application)->priv;
 
   if (priv->provider == NULL)
    {
@@ -272,10 +271,49 @@ gcal_application_startup (GApplication *app)
                                                  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION + 2);
     }
 
-  priv->manager = gcal_manager_new_with_settings (priv->settings);
-  g_signal_connect (priv->manager, "source-added", G_CALLBACK (source_added_cb), app);
+  gcal_application_set_app_menu (application);
 
-  gcal_application_set_app_menu (app);
+  if (priv->window != NULL)
+    {
+      gtk_window_present (GTK_WINDOW (priv->window));
+      if (priv->initial_date != NULL)
+        g_object_set (priv->window, "active-date", priv->initial_date, NULL);
+    }
+  else
+    {
+      if (priv->initial_date == NULL)
+        {
+          priv->initial_date = g_new0 (icaltimetype, 1);
+          *(priv->initial_date) = icaltime_current_time_with_zone (gcal_manager_get_system_timezone (priv->manager));
+          *(priv->initial_date) = icaltime_set_timezone (priv->initial_date,
+                                                         gcal_manager_get_system_timezone (priv->manager));
+        }
+
+      priv->window = gcal_window_new_with_view_and_date (GCAL_APPLICATION (application),
+                                                         g_settings_get_enum (priv->settings, "active-view"),
+                                                         priv->initial_date);
+      g_settings_bind (priv->settings, "active-view", priv->window, "active-view",
+                       G_SETTINGS_BIND_SET | G_SETTINGS_BIND_GET);
+
+      gtk_widget_show (priv->window);
+    }
+
+    g_clear_pointer (&(priv->initial_date), g_free);
+    if (priv->uuid != NULL)
+      {
+        gcal_window_open_event_by_uuid (GCAL_WINDOW (priv->window), priv->uuid);
+        g_clear_pointer (&(priv->uuid), g_free);
+      }
+}
+
+static void
+gcal_application_startup (GApplication *app)
+{
+  G_APPLICATION_CLASS (gcal_application_parent_class)->startup (app);
+
+  /* We're assuming the application is called as a service only by the shell search system */
+  if ((g_application_get_flags (app) & G_APPLICATION_IS_SERVICE) != 0)
+    g_application_set_inactivity_timeout (app, 3 * 60 * 1000);
 }
 
 static gint
@@ -289,7 +327,7 @@ gcal_application_command_line (GApplication            *app,
   GError *error;
   gint argc;
 
-  priv = gcal_application_get_instance_private (GCAL_APPLICATION (app));
+  priv = GCAL_APPLICATION (app)->priv;
   argv = g_application_command_line_get_arguments (command_line, &argc);
   context = g_option_context_new (N_("- Calendar management"));
   g_option_context_add_main_entries (context, gcal_application_goptions, GETTEXT_PACKAGE);
@@ -309,7 +347,12 @@ gcal_application_command_line (GApplication            *app,
       return 0;
     }
 
-  if (date != NULL)
+  if (uuid != NULL)
+    {
+      gcal_application_set_uuid (GCAL_APPLICATION (app), uuid);
+      g_clear_pointer (&uuid, g_free);
+    }
+  else if (date != NULL)
     {
       struct tm result;
 
@@ -321,7 +364,6 @@ gcal_application_command_line (GApplication            *app,
 
           *(priv->initial_date) = icaltime_set_timezone (priv->initial_date,
                                                          gcal_manager_get_system_timezone (priv->manager));
-          print_date ("loading date", priv->initial_date);
         }
 
       g_clear_pointer (&date, g_free);
@@ -335,6 +377,50 @@ gcal_application_command_line (GApplication            *app,
   return 0;
 }
 
+static gboolean
+gcal_application_dbus_register (GApplication    *application,
+                                GDBusConnection *connection,
+                                const gchar     *object_path,
+                                GError         **error)
+{
+  GcalApplicationPrivate *priv;
+  gchar *search_provider_path = NULL;
+  gboolean ret_val = FALSE;
+
+  priv = GCAL_APPLICATION (application)->priv;
+
+  if (!G_APPLICATION_CLASS (gcal_application_parent_class)->dbus_register (application, connection, object_path, error))
+    goto out;
+
+  search_provider_path = g_strconcat (object_path, "/SearchProvider", NULL);
+  if (!gcal_shell_search_provider_dbus_export (priv->search_provider, connection, search_provider_path, error))
+    goto out;
+
+  ret_val = TRUE;
+
+out:
+  g_free (search_provider_path);
+  return ret_val;
+}
+
+static void
+gcal_application_dbus_unregister (GApplication *application,
+                                  GDBusConnection *connection,
+                                  const gchar *object_path)
+{
+  GcalApplicationPrivate *priv;
+  gchar *search_provider_path = NULL;
+
+  priv = GCAL_APPLICATION (application)->priv;
+
+  search_provider_path = g_strconcat (object_path, "/SearchProvider", NULL);
+  gcal_shell_search_provider_dbus_unexport (priv->search_provider, connection, search_provider_path);
+
+  G_APPLICATION_CLASS (gcal_application_parent_class)->dbus_unregister (application, connection, object_path);
+
+  g_free (search_provider_path);
+}
+
 static void
 gcal_application_set_app_menu (GApplication *app)
 {
@@ -361,10 +447,7 @@ gcal_application_create_new_event (GSimpleAction *new_event,
                                    GVariant      *parameter,
                                    gpointer       app)
 {
-  GcalApplicationPrivate *priv;
-
-  priv = gcal_application_get_instance_private (GCAL_APPLICATION (app));
-
+  GcalApplicationPrivate *priv = GCAL_APPLICATION (app)->priv;
   gcal_window_new_event (GCAL_WINDOW (priv->window));
 }
 
@@ -373,10 +456,7 @@ gcal_application_sync (GSimpleAction *sync,
                        GVariant      *parameter,
                        gpointer       app)
 {
-  GcalApplicationPrivate *priv;
-
-  priv = gcal_application_get_instance_private (GCAL_APPLICATION (app));
-
+  GcalApplicationPrivate *priv = GCAL_APPLICATION (app)->priv;
   gcal_manager_refresh (priv->manager);
 }
 
@@ -385,10 +465,7 @@ gcal_application_launch_search (GSimpleAction *search,
                                 GVariant      *parameter,
                                 gpointer       app)
 {
-  GcalApplicationPrivate *priv;
-
-  priv = gcal_application_get_instance_private (GCAL_APPLICATION (app));
-
+  GcalApplicationPrivate *priv = GCAL_APPLICATION (app)->priv;
   gcal_window_set_search_mode (GCAL_WINDOW (priv->window), TRUE);
 }
 
@@ -414,7 +491,7 @@ gcal_application_show_about (GSimpleAction *simple,
     NULL
   };
 
-  priv = gcal_application_get_instance_private (GCAL_APPLICATION (user_data));
+  priv = GCAL_APPLICATION (user_data)->priv;
   date = g_date_time_new_now_local ();
 
   if (g_date_time_get_year (date) == created_year)
@@ -449,10 +526,7 @@ gcal_application_quit (GSimpleAction *simple,
                        GVariant      *parameter,
                        gpointer       user_data)
 {
-  GcalApplicationPrivate *priv;
-
-  priv =
-    gcal_application_get_instance_private (GCAL_APPLICATION (user_data));
+  GcalApplicationPrivate *priv = GCAL_APPLICATION (user_data)->priv;
 
   gtk_widget_destroy (priv->window);
 }
@@ -472,17 +546,31 @@ gcal_application_new (void)
 GcalManager*
 gcal_application_get_manager (GcalApplication *app)
 {
-  GcalApplicationPrivate *priv;
-
-  priv = gcal_application_get_instance_private (app);
-  return priv->manager;
+  return app->priv->manager;
 }
 
 GSettings*
 gcal_application_get_settings (GcalApplication *app)
 {
-  GcalApplicationPrivate *priv;
+  return app->priv->settings;
+}
 
-  priv = gcal_application_get_instance_private (app);
-  return priv->settings;
+void
+gcal_application_set_uuid (GcalApplication *application,
+                           const gchar     *uuid)
+{
+  GcalApplicationPrivate *priv = application->priv;
+
+  g_free (priv->uuid);
+  priv->uuid = g_strdup (uuid);
+}
+
+void
+gcal_application_set_initial_date (GcalApplication *application,
+                                   const icaltimetype *date)
+{
+  GcalApplicationPrivate *priv = application->priv;
+
+  g_free (priv->initial_date);
+  priv->initial_date = gcal_dup_icaltime (date);
 }
