@@ -20,6 +20,8 @@
 #include "gcal-manager.h"
 #include "gcal-utils.h"
 
+#include <libedataserverui/libedataserverui.h>
+
 typedef struct
 {
   ESource       *source;
@@ -27,6 +29,16 @@ typedef struct
   GcalManager   *manager;
 } AsyncOpsData;
 
+typedef struct
+{
+  ECalDataModelSubscriber *subscriber;
+  gchar                   *query;
+
+  guint                    sources_left;
+  gboolean                 passed_start;
+  gboolean                 search_done;
+} ViewStateData;
+
 typedef struct
 {
   ECalClient     *client;
@@ -45,13 +57,20 @@ typedef struct
   GHashTable      *clients;
 
   ESourceRegistry *source_registry;
+  ECredentialsPrompter *credentials_prompter;
 
   ECalDataModel   *e_data_model;
   ECalDataModel   *search_data_model;
 
+  ECalDataModel   *shell_search_data_model;
+  ViewStateData   *search_view_data;
+
   GCancellable    *async_ops;
 
+  /* state flags */
   gchar          **disabled_sources;
+  gint             sources_at_launch;
+
   /* timezone */
   icaltimezone    *system_timezone;
 
@@ -81,6 +100,8 @@ enum
   SOURCE_ACTIVATED,
   SOURCE_ADDED,
   SOURCE_REMOVED,
+  LOAD_COMPLETED,
+  QUERY_COMPLETED,
   NUM_SIGNALS
 };
 
@@ -274,7 +295,7 @@ load_source (GcalManager *manager,
     {
       /* NULL: because maybe the operation cannot be really cancelled */
       e_cal_client_connect (source,
-                            E_CAL_CLIENT_SOURCE_TYPE_EVENTS, NULL,
+                            E_CAL_CLIENT_SOURCE_TYPE_EVENTS, 1, NULL,
                             on_client_connected,
                             manager);
     }
@@ -302,6 +323,10 @@ on_client_connected (GObject      *source_object,
   priv = gcal_manager_get_instance_private (GCAL_MANAGER (user_data));
   source = e_client_get_source (E_CLIENT (source_object));
 
+  priv->sources_at_launch--;
+  if (priv->sources_at_launch == 0)
+    g_signal_emit (user_data, signals[LOAD_COMPLETED], 0);
+
   error = NULL;
   client = E_CAL_CLIENT (e_cal_client_connect_finish (result, &error));
   if (error == NULL)
@@ -329,6 +354,8 @@ on_client_connected (GObject      *source_object,
 
           e_cal_data_model_add_client (priv->e_data_model, client);
           e_cal_data_model_add_client (priv->search_data_model, client);
+          if (priv->shell_search_data_model != NULL)
+            e_cal_data_model_add_client (priv->shell_search_data_model, client);
         }
 
       /* refresh client when it's added */
@@ -499,6 +526,114 @@ remove_source (GcalManager  *manager,
   g_signal_emit (manager, signals[SOURCE_REMOVED], 0, source);
 }
 
+static void
+model_state_changed (GcalManager            *manager,
+                     ECalClientView         *view,
+                     ECalDataModelViewState  state,
+                     guint                   percent,
+                     const gchar            *message,
+                     const GError           *error,
+                     ECalDataModel          *data_model)
+{
+  GcalManagerPrivate *priv = gcal_manager_get_instance_private (manager);
+  gchar *filter = e_cal_data_model_dup_filter (data_model);
+  if (state == E_CAL_DATA_MODEL_VIEW_STATE_START && g_strcmp0 (priv->search_view_data->query, filter) == 0)
+    {
+      priv->search_view_data->passed_start = TRUE;
+      goto out;
+    }
+
+  if (priv->search_view_data->passed_start && state == E_CAL_DATA_MODEL_VIEW_STATE_COMPLETE &&
+      g_strcmp0 (priv->search_view_data->query, filter) == 0)
+    {
+      priv->search_view_data->sources_left--;
+      priv->search_view_data->search_done = (priv->search_view_data->sources_left == 0);
+      if (priv->search_view_data->search_done)
+        g_signal_emit (manager, signals[QUERY_COMPLETED], 0);
+    }
+
+out:
+    g_free (filter);
+    return;
+}
+
+static void
+show_source_error (const gchar  *where,
+                   const gchar  *what,
+                   ESource      *source,
+                   const GError *error)
+{
+  if (!error || g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+    return;
+
+  /* TODO Show the error in UI, somehow */
+  g_warning ("%s: %s '%s': %s", where, what, e_source_get_display_name (source), error->message);
+}
+
+static void
+source_invoke_authenticate_cb (GObject      *source_object,
+                               GAsyncResult *result,
+                               gpointer      user_data)
+{
+  ESource *source = E_SOURCE (source_object);
+  GError *error = NULL;
+
+  if (!e_source_invoke_authenticate_finish (source, result, &error) &&
+      !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+    show_source_error (G_STRFUNC, "Failed to invoke authenticate", source, error);
+  }
+
+  g_clear_error (&error);
+}
+
+static void
+source_trust_prompt_done_cb (GObject      *source_object,
+                             GAsyncResult *result,
+                             gpointer      user_data)
+{
+  ETrustPromptResponse response = E_TRUST_PROMPT_RESPONSE_UNKNOWN;
+  ESource *source = E_SOURCE (source_object);
+  GError *error = NULL;
+
+  if (!e_trust_prompt_run_for_source_finish (source, result, &response, &error)) {
+    show_source_error (G_STRFUNC, "Failed to prompt for trust for", source, error);
+  } else if (response == E_TRUST_PROMPT_RESPONSE_ACCEPT || response == E_TRUST_PROMPT_RESPONSE_ACCEPT_TEMPORARILY) {
+    /* Use NULL credentials to reuse those from the last time. */
+    e_source_invoke_authenticate (source, NULL, NULL /* cancellable */, source_invoke_authenticate_cb, NULL);
+  }
+
+  g_clear_error (&error);
+}
+
+static void
+source_credentials_required_cb (ESourceRegistry         *registry,
+                                ESource                 *source,
+                                ESourceCredentialsReason reason,
+                                const gchar             *certificate_pem,
+                                GTlsCertificateFlags     certificate_errors,
+                                const GError            *op_error,
+                                GcalManager             *manager)
+{
+  GcalManagerPrivate *priv;
+  ECredentialsPrompter *credentials_prompter;
+
+  g_return_if_fail (GCAL_IS_MANAGER (manager));
+
+  priv = gcal_manager_get_instance_private (manager);
+  credentials_prompter = priv->credentials_prompter;
+
+  if (e_credentials_prompter_get_auto_prompt_disabled_for (credentials_prompter, source))
+    return;
+
+  if (reason == E_SOURCE_CREDENTIALS_REASON_SSL_FAILED) {
+    e_trust_prompt_run_for_source (e_credentials_prompter_get_dialog_parent (credentials_prompter),
+                                   source, certificate_pem, certificate_errors, op_error ? op_error->message : NULL,
+                                   TRUE /* allow_source_save */, NULL /* cancellable */, source_trust_prompt_done_cb, NULL);
+  } else if (reason == E_SOURCE_CREDENTIALS_REASON_ERROR && op_error) {
+    show_source_error (G_STRFUNC, "Failed to authenticate", source, op_error);
+  }
+}
+
 static void
 gcal_manager_class_init (GcalManagerClass *klass)
 {
@@ -530,6 +665,16 @@ gcal_manager_class_init (GcalManagerClass *klass)
                                           G_STRUCT_OFFSET (GcalManagerClass, source_removed),
                                           NULL, NULL, NULL,
                                           G_TYPE_NONE, 1, G_TYPE_POINTER);
+
+  signals[LOAD_COMPLETED] = g_signal_new ("load-completed", GCAL_TYPE_MANAGER, G_SIGNAL_RUN_LAST,
+                                          G_STRUCT_OFFSET (GcalManagerClass, load_completed),
+                                          NULL, NULL, NULL,
+                                          G_TYPE_NONE, 0);
+
+  signals[QUERY_COMPLETED] = g_signal_new ("query-completed", GCAL_TYPE_MANAGER, G_SIGNAL_RUN_LAST,
+                                           G_STRUCT_OFFSET (GcalManagerClass, query_completed),
+                                           NULL, NULL, NULL,
+                                           G_TYPE_NONE, 0);
 }
 
 static void
@@ -545,6 +690,7 @@ gcal_manager_constructed (GObject *object)
 
   GList *sources, *l;
   GError *error = NULL;
+  ESourceCredentialsProvider *credentials_provider;
 
   G_OBJECT_CLASS (gcal_manager_parent_class)->constructed (object);
 
@@ -565,10 +711,45 @@ gcal_manager_constructed (GObject *object)
       return;
     }
 
-  sources = e_source_registry_list_enabled (priv->source_registry, E_SOURCE_EXTENSION_CALENDAR);
-  for (l = sources; l != NULL; l = l->next)
-    load_source (GCAL_MANAGER (object), l->data);
-  g_list_free (sources);
+  priv->credentials_prompter = e_credentials_prompter_new (priv->source_registry);
+
+  /* First disable credentials prompt for all but calendar sources... */
+  sources = e_source_registry_list_sources (priv->source_registry, NULL);
+
+  for (l = sources; l != NULL; l = g_list_next (l))
+    {
+      ESource *source = E_SOURCE (l->data);
+
+      /* Mark for skip also currently disabled sources */
+      if (!e_source_has_extension (source, E_SOURCE_EXTENSION_CALENDAR))
+        e_credentials_prompter_set_auto_prompt_disabled_for (priv->credentials_prompter, source, TRUE);
+    }
+
+  g_list_free_full (sources, g_object_unref);
+
+  credentials_provider = e_credentials_prompter_get_provider (priv->credentials_prompter);
+
+  /* ...then enable credentials prompt for credential source of the calendar sources,
+     which can be a collection source.  */
+  sources = e_source_registry_list_sources (priv->source_registry, E_SOURCE_EXTENSION_CALENDAR);
+
+  for (l = sources; l != NULL; l = g_list_next (l))
+    {
+      ESource *source = E_SOURCE (l->data), *cred_source;
+
+      cred_source = e_source_credentials_provider_ref_credentials_source (credentials_provider, source);
+      if (cred_source && !e_source_equal (source, cred_source))
+        e_credentials_prompter_set_auto_prompt_disabled_for (priv->credentials_prompter, cred_source, FALSE);
+      g_clear_object (&cred_source);
+    }
+
+  g_list_free_full (sources, g_object_unref);
+
+  /* The eds_credentials_prompter responses to REQUIRED and REJECTED reasons,
+     the SSL_FAILED should be handled elsewhere. */
+  g_signal_connect (priv->source_registry, "credentials-required", G_CALLBACK (source_credentials_required_cb), object);
+
+  e_credentials_prompter_process_awaiting_credentials (priv->credentials_prompter);
 
   g_signal_connect_swapped (priv->source_registry, "source-added", G_CALLBACK (load_source), object);
   g_signal_connect_swapped (priv->source_registry, "source-removed", G_CALLBACK (remove_source), object);
@@ -581,6 +762,13 @@ gcal_manager_constructed (GObject *object)
   e_cal_data_model_set_timezone (priv->e_data_model, priv->system_timezone);
   e_cal_data_model_set_expand_recurrences (priv->search_data_model, TRUE);
   e_cal_data_model_set_timezone (priv->search_data_model, priv->system_timezone);
+
+  sources = e_source_registry_list_enabled (priv->source_registry, E_SOURCE_EXTENSION_CALENDAR);
+  priv->sources_at_launch = g_list_length (sources);
+
+  for (l = sources; l != NULL; l = l->next)
+    load_source (GCAL_MANAGER (object), l->data);
+  g_list_free (sources);
 }
 
 static void
@@ -599,6 +787,14 @@ gcal_manager_finalize (GObject *object)
   if (priv->search_data_model != NULL)
     g_object_unref (priv->search_data_model);
 
+  if (priv->search_view_data != NULL)
+    {
+      g_free (priv->search_view_data->query);
+      g_free (priv->search_view_data);
+    }
+  if (priv->shell_search_data_model != NULL)
+    g_object_unref (priv->shell_search_data_model);
+
   g_hash_table_destroy (priv->clients);
 }
 
@@ -730,6 +926,84 @@ gcal_manager_get_system_timezone (GcalManager *manager)
   return priv->system_timezone;
 }
 
+void
+gcal_manager_setup_shell_search (GcalManager             *manager,
+                                 ECalDataModelSubscriber *subscriber)
+{
+  GcalManagerPrivate *priv;
+  priv = gcal_manager_get_instance_private (manager);
+
+  if (priv->shell_search_data_model == NULL)
+    {
+      priv->shell_search_data_model = e_cal_data_model_new (submit_thread_job);
+      g_signal_connect_swapped (priv->shell_search_data_model, "view-state-changed", G_CALLBACK (model_state_changed), manager);
+
+      e_cal_data_model_set_expand_recurrences (priv->shell_search_data_model, TRUE);
+      e_cal_data_model_set_timezone (priv->shell_search_data_model, priv->system_timezone);
+
+      priv->search_view_data = g_new0 (ViewStateData, 1);
+      priv->search_view_data->subscriber = subscriber;
+    }
+}
+
+/**
+ * gcal_manager_set_shell_search_query:
+ * @manager: A #GcalManager instance
+ * @query: query string (an s-exp)
+ *
+ * Set the query terms of the #ECalDataModel used in the shell search
+ **/
+void
+gcal_manager_set_shell_search_query (GcalManager *manager,
+                                     const gchar *query)
+{
+  GcalManagerPrivate *priv = gcal_manager_get_instance_private (manager);
+
+  priv->search_view_data->passed_start = FALSE;
+  priv->search_view_data->search_done = FALSE;
+  priv->search_view_data->sources_left = g_hash_table_size (priv->clients) - g_strv_length (priv->disabled_sources);
+
+  if (priv->search_view_data->query != NULL)
+    g_free (priv->search_view_data->query);
+  priv->search_view_data->query = g_strdup (query);
+
+  e_cal_data_model_set_filter (priv->shell_search_data_model, query);
+}
+
+void
+gcal_manager_set_shell_search_subscriber (GcalManager             *manager,
+                                          ECalDataModelSubscriber *subscriber,
+                                          time_t                   range_start,
+                                          time_t                   range_end)
+{
+  GcalManagerPrivate *priv;
+
+  priv = gcal_manager_get_instance_private (manager);
+  e_cal_data_model_subscribe (priv->shell_search_data_model, subscriber, range_start, range_end);
+}
+
+gboolean
+gcal_manager_shell_search_done (GcalManager *manager)
+{
+  GcalManagerPrivate *priv = gcal_manager_get_instance_private (manager);
+  return priv->search_view_data->search_done;
+}
+
+GList*
+gcal_manager_get_shell_search_events (GcalManager *manager)
+{
+  GcalManagerPrivate *priv;
+  time_t range_start, range_end;
+  GList *list = NULL;
+
+  priv = gcal_manager_get_instance_private (manager);
+
+  e_cal_data_model_get_subscriber_range (priv->shell_search_data_model, priv->search_view_data->subscriber,
+                                         &range_start, &range_end);
+  e_cal_data_model_foreach_component (priv->shell_search_data_model, range_start, range_end, gather_components, &list);
+  return list;
+}
+
 void
 gcal_manager_set_subscriber (GcalManager             *manager,
                              ECalDataModelSubscriber *subscriber,
@@ -860,6 +1134,8 @@ gcal_manager_enable_source (GcalManager *manager,
   unit->enabled = TRUE;
   e_cal_data_model_add_client (priv->e_data_model, unit->client);
   e_cal_data_model_add_client (priv->search_data_model, unit->client);
+  if (priv->shell_search_data_model != NULL)
+    e_cal_data_model_add_client (priv->shell_search_data_model, unit->client);
 
   /* remove source's uid from disabled_sources array */
   new_disabled_sources = g_new0 (gchar*, g_strv_length (priv->disabled_sources));
@@ -891,6 +1167,7 @@ gcal_manager_disable_source (GcalManager *manager,
   GcalManagerUnit *unit;
   gchar **new_disabled_sources;
   gint i;
+  const gchar *source_uid;
 
   priv = gcal_manager_get_instance_private (manager);
   unit = g_hash_table_lookup (priv->clients, source);
@@ -898,16 +1175,19 @@ gcal_manager_disable_source (GcalManager *manager,
   if (!unit->enabled)
     return;
 
+  source_uid = e_source_get_uid (source);
   unit->enabled = FALSE;
-  e_cal_data_model_remove_client (priv->e_data_model, e_source_get_uid (source));
-  e_cal_data_model_remove_client (priv->search_data_model, e_source_get_uid (source));
+  e_cal_data_model_remove_client (priv->e_data_model, source_uid);
+  e_cal_data_model_remove_client (priv->search_data_model, source_uid);
+  if (priv->shell_search_data_model != NULL)
+    e_cal_data_model_remove_client (priv->shell_search_data_model, source_uid);
 
   /* add source's uid from disabled_sources array */
   new_disabled_sources = g_new0 (gchar*, g_strv_length (priv->disabled_sources) + 2);
   for (i = 0; i < g_strv_length (priv->disabled_sources); i++)
-      new_disabled_sources[i] = g_strdup (priv->disabled_sources[i]);
+    new_disabled_sources[i] = g_strdup (priv->disabled_sources[i]);
 
-  new_disabled_sources[g_strv_length (priv->disabled_sources)] = e_source_dup_uid (source);
+  new_disabled_sources[g_strv_length (priv->disabled_sources)] = g_strdup (source_uid);
 
   g_strfreev (priv->disabled_sources);
   priv->disabled_sources = new_disabled_sources;
@@ -916,6 +1196,20 @@ gcal_manager_disable_source (GcalManager *manager,
   g_settings_set_strv (priv->settings, "disabled-sources", (const gchar * const *) priv->disabled_sources);
 }
 
+gboolean
+gcal_manager_source_enabled (GcalManager *manager,
+                             ESource     *source)
+{
+  GcalManagerPrivate *priv;
+  GcalManagerUnit *unit;
+
+  priv = gcal_manager_get_instance_private (manager);
+  unit = g_hash_table_lookup (priv->clients, source);
+  if (unit == NULL)
+    return FALSE;
+  return unit->enabled;
+}
+
 void
 gcal_manager_refresh (GcalManager *manager)
 {
@@ -1066,3 +1360,47 @@ gcal_manager_get_events (GcalManager  *manager,
   return list;
 }
 
+gboolean
+gcal_manager_load_completed (GcalManager *manager)
+{
+  GcalManagerPrivate *priv = gcal_manager_get_instance_private (manager);
+  return priv->sources_at_launch == 0;
+}
+
+GcalEventData*
+gcal_manager_get_event_from_shell_search (GcalManager *manager,
+                                          const gchar *uuid)
+{
+  GcalManagerPrivate *priv;
+  time_t range_start, range_end;
+  GList *l, *list = NULL;
+  GcalEventData *new_data, *data;
+  gchar *cuuid;
+
+  priv = gcal_manager_get_instance_private (manager);
+
+  e_cal_data_model_get_subscriber_range (priv->shell_search_data_model, priv->search_view_data->subscriber,
+                                         &range_start, &range_end);
+  e_cal_data_model_foreach_component (priv->shell_search_data_model, range_start, range_end, gather_components, &list);
+  if (list != NULL)
+    new_data = g_new0 (GcalEventData, 1);
+
+  for (l = list; l != NULL; l = g_list_next (l))
+    {
+      data = l->data;
+      cuuid = get_uuid_from_component (data->source, data->event_component);
+
+      if (g_strcmp0 (cuuid, uuid) == 0)
+        {
+          new_data->source = data->source;
+          new_data->event_component = g_object_ref (data->event_component);
+        }
+
+      g_object_unref (data->event_component);
+      g_free (data);
+      g_free (cuuid);
+    }
+  g_list_free (list);
+
+  return new_data;
+}
