@@ -55,7 +55,6 @@ typedef struct
    * These keeps the cell index, meaning 0 for left top, 1 for the next in the row from left to right, etc.
    * Note that this does not take into account the disabled row before the first active row.
    */
-  gint            clicked_cell;
   gint            start_mark_cell;
   gint            end_mark_cell;
 
@@ -73,9 +72,10 @@ typedef struct
   /* text direction factors */
   gint            k;
 
+  icaltimetype   *current_date;
+
   /* property */
   icaltimetype   *date;
-  GcalManager    *manager; /* weak reference */
 } GcalMonthViewPrivate;
 
 enum
@@ -344,13 +344,14 @@ rebuild_popover_for_day (GcalMonthView *view,
   GtkWidget *child_widget;
   GdkRectangle rect;
 
-  gchar str_date[64];
-  struct tm tm_date;
-  gchar *markup;
+  gchar *label_title;
+
+  icaltimetype date, second_date;
+  const icaltimetype *dt_start, *dt_end;
+  gint start_comparison, end_comparison;
 
   /* placement helpers */
-  gdouble start_grid_y;
-  gdouble cell_width, cell_height;
+  gdouble start_grid_y, cell_width, cell_height;
 
   gint shown_rows;
 
@@ -364,22 +365,44 @@ rebuild_popover_for_day (GcalMonthView *view,
   priv = gcal_month_view_get_instance_private (view);
   ppriv = GCAL_SUBSCRIBER_VIEW (view)->priv;
 
-  tm_date = icaltimetype_to_tm (priv->date);
-  e_utf8_strftime_fix_am_pm (str_date, 64, "%B", &tm_date);
-  markup = g_strdup_printf ("<span weight=\"bold\">%s %d</span>", str_date, day);
-  gtk_label_set_markup (GTK_LABEL (priv->popover_title), markup);
-  g_free (markup);
+  label_title = g_strdup_printf ("%s %d", gcal_get_month_name (priv->date->month - 1), day);
+  gtk_label_set_text (GTK_LABEL (priv->popover_title), label_title);
+  g_free (label_title);
 
   /* Clean all the widgets */
   gtk_container_foreach (GTK_CONTAINER (priv->events_list_box), (GtkCallback) gtk_widget_destroy, NULL);
 
   l = g_hash_table_lookup (ppriv->overflow_cells, GINT_TO_POINTER (priv->pressed_overflow_indicator));
+  if (l != NULL)
+    {
+      date = *(priv->date);
+      date.day = day;
+
+      second_date = date;
+
+      date.hour = 0;
+      date.minute = 0;
+      second_date.hour = 23;
+      second_date.minute =59;
+    }
   for (; l != NULL; l = g_list_next (l))
     {
-      /* FIXME: mark the widgets properly with CSS tags */
       child_widget = gcal_event_widget_clone (GCAL_EVENT_WIDGET (l->data));
       gtk_container_add (GTK_CONTAINER (priv->events_list_box), child_widget);
       _gcal_subscriber_view_setup_child (GCAL_SUBSCRIBER_VIEW (view), child_widget);
+
+      dt_start = gcal_event_widget_peek_start_date (GCAL_EVENT_WIDGET (child_widget));
+      dt_end = gcal_event_widget_peek_end_date (GCAL_EVENT_WIDGET (child_widget));
+
+      date.is_date = dt_start->is_date;
+      start_comparison = icaltime_compare (*dt_start, date);
+      end_comparison = icaltime_compare (second_date, *dt_end);
+      if (start_comparison == -1 && end_comparison == -1)
+        gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted");
+      else if (start_comparison == -1)
+        gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted-start");
+      else if (end_comparison == -1)
+        gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted-end");
     }
 
   /* placement calculation */
@@ -426,7 +449,7 @@ rebuild_popover_for_day (GcalMonthView *view,
 
   /* sizing hack */
   child_widget = gtk_bin_get_child (GTK_BIN (priv->overflow_popover));
-  gtk_widget_set_size_request (child_widget, cell_width, -1);
+  gtk_widget_set_size_request (child_widget, 200, -1);
 
   g_object_set_data (G_OBJECT (priv->overflow_popover), "selected-day", GINT_TO_POINTER (day));
 }
@@ -475,7 +498,7 @@ update_list_box_headers (GtkListBoxRow *row,
 
       label = gtk_label_new (time);
       gtk_style_context_add_class (gtk_widget_get_style_context (label), GTK_STYLE_CLASS_DIM_LABEL);
-      g_object_set (label, "margin-start", 6, "margin-top", 2, "halign", GTK_ALIGN_START, NULL);
+      g_object_set (label, "margin-start", 6, "halign", GTK_ALIGN_START, NULL);
 
       vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
       gtk_container_add (GTK_CONTAINER (vbox), label);
@@ -551,8 +574,6 @@ gcal_month_view_init (GcalMonthView *self)
 
   priv = gcal_month_view_get_instance_private (self);
 
-  priv->clicked_cell = -1;
-
   priv->start_mark_cell = -1;
   priv->end_mark_cell = -1;
 
@@ -567,19 +588,21 @@ gcal_month_view_init (GcalMonthView *self)
     priv->k = 1;
 
   priv->overflow_popover = gtk_popover_new (GTK_WIDGET (self));
+  gtk_style_context_add_class (gtk_widget_get_style_context (priv->overflow_popover), "events");
   g_signal_connect (priv->overflow_popover, "hide", G_CALLBACK (overflow_popover_hide), self);
 
   grid = gtk_grid_new ();
-  g_object_set (grid, "margin", 6, "row-spacing", 6, "orientation", GTK_ORIENTATION_VERTICAL, NULL);
+  g_object_set (grid, "row-spacing", 6, "orientation", GTK_ORIENTATION_VERTICAL, NULL);
   gtk_container_add (GTK_CONTAINER (priv->overflow_popover), grid);
 
   priv->popover_title = gtk_label_new (NULL);
+  gtk_style_context_add_class (gtk_widget_get_style_context (priv->popover_title), "sidebar-header");
+  g_object_set (priv->popover_title, "margin", 6, "halign", GTK_ALIGN_START, NULL);
   priv->events_list_box = gtk_list_box_new ();
   gtk_list_box_set_selection_mode (GTK_LIST_BOX (priv->events_list_box), GTK_SELECTION_NONE);
   gtk_list_box_set_header_func (GTK_LIST_BOX (priv->events_list_box), update_list_box_headers, self, NULL);
-  gtk_style_context_remove_class (gtk_widget_get_style_context (priv->events_list_box), GTK_STYLE_CLASS_LIST);
-  button = gtk_button_new_with_label (_("Add new event…"));
-  g_object_set (button, "margin", 6, "hexpand", TRUE, NULL);
+  button = gtk_button_new_with_label (_("Add Event…"));
+  g_object_set (button, "hexpand", TRUE, NULL);
   g_signal_connect (button, "clicked", G_CALLBACK (add_new_event_button_cb), self);
 
   gtk_container_add (GTK_CONTAINER (grid), priv->popover_title);
@@ -612,32 +635,11 @@ gcal_month_view_set_property (GObject       *object,
     {
     case PROP_DATE:
       {
-        time_t range_start, range_end;
-        icaltimetype *date;
-        icaltimezone* default_zone;
-
         if (priv->date != NULL)
           g_free (priv->date);
 
         priv->date = g_value_dup_boxed (value);
-
-        date = gcal_view_get_initial_date (GCAL_VIEW (object));
-        priv->days_delay = (icaltime_day_of_week (*date) - priv->first_weekday + 6) % 7;
-
-        default_zone =
-          gcal_manager_get_system_timezone (priv->manager);
-        range_start = icaltime_as_timet_with_zone (*date,
-                                                   default_zone);
-        g_free (date);
-        date = gcal_view_get_final_date (GCAL_VIEW (object));
-        range_end = icaltime_as_timet_with_zone (*date,
-                                                 default_zone);
-        g_free (date);
-
-        gcal_manager_set_subscriber (priv->manager,
-                                     E_CAL_DATA_MODEL_SUBSCRIBER (object),
-                                     range_start,
-                                     range_end);
+        priv->days_delay = (time_day_of_week (1, priv->date->month - 1, priv->date->year) - priv->first_weekday + 7) % 7;
         gtk_widget_queue_draw (GTK_WIDGET (object));
         break;
       }
@@ -770,6 +772,7 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
 
   gint i, j, sw, shown_rows;
 
+  GtkBorder margin;
   gint padding_bottom, font_height;
   PangoLayout *layout;
   PangoFontDescription *font_desc;
@@ -842,7 +845,7 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
   for (l = ppriv->multi_cell_children; l != NULL; l = g_list_next (l))
     {
       gint first_cell, last_cell, first_row, last_row, start, end;
-      gboolean visible;
+      gboolean visible, start_before = TRUE, end_after = TRUE;
 
       const icaltimetype *date;
       GArray *cells, *lengths;
@@ -858,7 +861,10 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
       j = 1;
       date = gcal_event_widget_peek_start_date (GCAL_EVENT_WIDGET (child_widget));
       if (date->month == priv->date->month)
-        j = date->day;
+        {
+          j = date->day;
+          start_before = FALSE;
+        }
       j += priv->days_delay;
       first_cell = 7 * ((j - 1) / 7)+ 6 * priv->k + sw * ((j - 1) % 7);
 
@@ -867,18 +873,22 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
       if (gcal_event_widget_get_all_day (GCAL_EVENT_WIDGET (child_widget)))
         {
           if (date->month == priv->date->month)
-            j = date->day - 1;
+            {
+              j = date->day - 1;
+              end_after = FALSE;
+            }
         }
       else
         {
           if (date->month == priv->date->month)
-            j = date->day;
+            {
+              j = date->day;
+              end_after = FALSE;
+            }
         }
       j += priv->days_delay;
       last_cell = 7 * ((j - 1) / 7)+ 6 * priv->k + sw * ((j - 1) % 7);
 
-      /* FIXME missing mark widgets with continuos tags */
-
       first_row = first_cell / 7;
       last_row = last_cell / 7;
       visible = TRUE;
@@ -908,13 +918,6 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
               gint cell_idx = g_array_index (cells, gint, i);
               gint row = cell_idx / 7;
               gint column = cell_idx % 7;
-              pos_x = cell_width * column;
-              pos_y = cell_height * (row + first_row_gap) + start_grid_y;
-
-              child_allocation.x = pos_x;
-              child_allocation.y = pos_y + vertical_cell_space - size_left[cell_idx];
-              child_allocation.width = cell_width * g_array_index (lengths, gint, i);
-              child_allocation.height = natural_height;
 
               if (i != 0)
                 {
@@ -923,15 +926,47 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
                   _gcal_subscriber_view_setup_child (GCAL_SUBSCRIBER_VIEW (widget), child_widget);
                   gtk_widget_show (child_widget);
 
+                  if (i != cells->len - 1 || end_after)
+                    gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted");
+                  else
+                    gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted-start");
+
                   aux = g_hash_table_lookup (ppriv->children, uuid);
                   aux = g_list_append (aux, child_widget);
                 }
+              else if (i == 0 && cells->len == 1)
+                {
+                  if (start_before)
+                    gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted-start");
+                  if (end_after)
+                    gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted-end");
+                }
+              else if (i != cells->len - 1)
+                {
+                  if (start_before)
+                    gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted");
+                  else
+                    gtk_style_context_add_class (gtk_widget_get_style_context (child_widget), "slanted-end");
+                }
+
+              gtk_style_context_get_margin (gtk_widget_get_style_context (child_widget),
+                                            gtk_widget_get_state_flags (child_widget),
+                                            &margin);
+
+              pos_x = cell_width * column + margin.left;
+              pos_y = cell_height * (row + first_row_gap) + start_grid_y + margin.top;
+
+              child_allocation.x = pos_x;
+              child_allocation.y = pos_y + vertical_cell_space - size_left[cell_idx];
+              child_allocation.width = cell_width * g_array_index (lengths, gint, i) - (margin.left + margin.right);
+              child_allocation.height = natural_height;
+
               gtk_widget_size_allocate (child_widget, &child_allocation);
               g_hash_table_remove (ppriv->hidden_as_overflow, uuid);
 
               /* update size_left */
               for (j = 0; j < g_array_index (lengths, gint, i); j++)
-                size_left[cell_idx + j] -= natural_height;
+                size_left[cell_idx + j] -= natural_height + margin.top + margin.bottom;
             }
         }
       else
@@ -976,18 +1011,22 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
 
           if (size_left[i] > natural_height)
             {
-              pos_x = cell_width * (i % 7);
-              pos_y = cell_height * ((i / 7) + first_row_gap) + start_grid_y;
+              gtk_style_context_get_margin (gtk_widget_get_style_context (child_widget),
+                                            gtk_widget_get_state_flags (child_widget),
+                                            &margin);
+
+              pos_x = cell_width * (i % 7) + margin.left;
+              pos_y = cell_height * ((i / 7) + first_row_gap) + start_grid_y + margin.top;
 
               child_allocation.x = pos_x;
               child_allocation.y = pos_y + vertical_cell_space - size_left[i];
-              child_allocation.width = cell_width;
+              child_allocation.width = cell_width - (margin.left + margin.right);
               child_allocation.height = natural_height;
               gtk_widget_show (child_widget);
               gtk_widget_size_allocate (child_widget, &child_allocation);
               g_hash_table_remove (ppriv->hidden_as_overflow, uuid);
 
-              size_left[i] -= natural_height;
+              size_left[i] -= natural_height + margin.top + margin.bottom;
             }
           else
             {
@@ -1005,9 +1044,8 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
         }
     }
 
-  /* FIXME: remove on Gtk+ 3.15.4 release */
   if (g_hash_table_size (ppriv->overflow_cells) != 0)
-    gtk_widget_queue_draw_area (widget, allocation->x, allocation->y, allocation->width, allocation->height);
+    gtk_widget_queue_draw (widget);
 
   ppriv->children_changed = FALSE;
 }
@@ -1208,7 +1246,8 @@ gcal_month_view_draw (GtkWidget *widget,
           g_object_unref (overflow_layout);
         }
 
-      if (priv->date->day == j)
+      if (priv->date->year == priv->current_date->year && priv->date->month == priv->current_date->month &&
+          j == priv->current_date->day)
         {
           PangoLayout *clayout;
           PangoFontDescription *cfont_desc;
@@ -1378,8 +1417,7 @@ gcal_month_view_button_press (GtkWidget      *widget,
   GcalSubscriberViewPrivate *ppriv;
   GcalMonthViewPrivate *priv;
 
-  gint days;
-  gint j, sw;
+  gint days, j, sw, clicked_cell;
   gboolean pressed_indicator = FALSE;
 
   priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (widget));
@@ -1388,19 +1426,16 @@ gcal_month_view_button_press (GtkWidget      *widget,
   days = priv->days_delay + icaltime_days_in_month (priv->date->month, priv->date->year);
   sw = 1 - 2 * priv->k;
 
-  priv->clicked_cell = gather_button_event_data (GCAL_MONTH_VIEW (widget), event->x, event->y,
+  clicked_cell = gather_button_event_data (GCAL_MONTH_VIEW (widget), event->x, event->y,
                                                  &pressed_indicator, NULL, NULL);
 
-  j = 7 * ((priv->clicked_cell + 7 * priv->k) / 7) + sw * (priv->clicked_cell % 7) + (1 - priv->k);
+  j = 7 * ((clicked_cell + 7 * priv->k) / 7) + sw * (clicked_cell % 7) + (1 - priv->k);
 
   if (j > priv->days_delay && j <= days)
-    priv->start_mark_cell = priv->clicked_cell;
+    priv->start_mark_cell = clicked_cell;
 
-  if (pressed_indicator && g_hash_table_contains (ppriv->overflow_cells, GINT_TO_POINTER (priv->clicked_cell)))
-    priv->pressed_overflow_indicator = priv->clicked_cell;
-
-  g_debug ("clicked is: %d", priv->clicked_cell);
-  g_debug ("pressed is: %d", priv->start_mark_cell);
+  if (pressed_indicator && g_hash_table_contains (ppriv->overflow_cells, GINT_TO_POINTER (clicked_cell)))
+    priv->pressed_overflow_indicator = clicked_cell;
 
   return TRUE;
 }
@@ -1435,7 +1470,7 @@ gcal_month_view_motion_notify_event (GtkWidget      *widget,
 
   j = 7 * ((new_end_cell + 7 * priv->k) / 7) + sw * (new_end_cell % 7) + (1 - priv->k);
 
-  if (priv->clicked_cell != -1)
+  if (priv->start_mark_cell != -1)
     {
       if (j > priv->days_delay && j <= days)
         {
@@ -1444,7 +1479,6 @@ gcal_month_view_motion_notify_event (GtkWidget      *widget,
           if (priv->end_mark_cell != new_end_cell)
             gtk_widget_queue_draw (widget);
 
-          g_debug ("move_notify: %d, %d, %d", priv->start_mark_cell, priv->end_mark_cell, new_end_cell);
           priv->end_mark_cell = new_end_cell;
           return TRUE;
         }
@@ -1491,7 +1525,7 @@ gcal_month_view_button_release (GtkWidget      *widget,
   priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (widget));
   ppriv = GCAL_SUBSCRIBER_VIEW (widget)->priv;
 
-  if (priv->clicked_cell == -1)
+  if (priv->start_mark_cell == -1)
     return FALSE;
 
   days = priv->days_delay + icaltime_days_in_month (priv->date->month, priv->date->year);
@@ -1503,7 +1537,6 @@ gcal_month_view_button_release (GtkWidget      *widget,
 
   if (j <= priv->days_delay || j > days)
     {
-      priv->clicked_cell = -1;
       priv->pressed_overflow_indicator = -1;
       priv->start_mark_cell = -1;
       priv->end_mark_cell = -1;
@@ -1513,7 +1546,6 @@ gcal_month_view_button_release (GtkWidget      *widget,
     }
 
   priv->end_mark_cell = released;
-  g_debug ("released button cell: %d", priv->end_mark_cell);
 
   if (priv->pressed_overflow_indicator != -1 && priv->start_mark_cell == priv->end_mark_cell &&
       g_hash_table_contains (ppriv->overflow_cells, GINT_TO_POINTER (priv->pressed_overflow_indicator)))
@@ -1524,7 +1556,6 @@ gcal_month_view_button_release (GtkWidget      *widget,
       gtk_widget_show_all (priv->overflow_popover);
 
       gtk_widget_queue_draw (widget);
-      priv->clicked_cell = -1;
       priv->pressed_overflow_indicator = -1;
       priv->start_mark_cell = -1;
       priv->end_mark_cell = -1;
@@ -1538,7 +1569,7 @@ gcal_month_view_button_release (GtkWidget      *widget,
       start_date->is_date = 1;
 
       end_date = gcal_dup_icaltime (priv->date);
-      end_date->day = j - priv->days_delay + 1;
+      end_date->day = j - priv->days_delay;
       end_date->is_date = 1;
 
       if (start_date->day > end_date->day)
@@ -1547,6 +1578,7 @@ gcal_month_view_button_release (GtkWidget      *widget,
           start_date->day = end_date->day;
           end_date->day = day;
         }
+      end_date->day += 1 ;
 
       g_signal_emit_by_name (GCAL_VIEW (widget), "create-event", start_date, end_date, x, y);
 
@@ -1555,7 +1587,6 @@ gcal_month_view_button_release (GtkWidget      *widget,
     }
 
   gtk_widget_queue_draw (widget);
-  priv->clicked_cell = -1;
   priv->pressed_overflow_indicator = -1;
   return TRUE;
 }
@@ -1684,27 +1715,14 @@ gcal_month_view_get_children_by_uuid (GcalView    *view,
 }
 
 /* Public API */
-/**
- * gcal_month_view_new:
- *
- * Since: 0.1
- * Create a new month view widget
- *
- * Returns: (transfer full):
- **/
-GtkWidget*
-gcal_month_view_new (void)
-{
-  return g_object_new (GCAL_TYPE_MONTH_VIEW, NULL);
-}
-
 void
-gcal_month_view_set_manager (GcalMonthView *month_view,
-                             GcalManager   *manager)
+gcal_month_view_set_current_date (GcalMonthView *month_view,
+                                  icaltimetype  *current_date)
 {
   GcalMonthViewPrivate *priv = gcal_month_view_get_instance_private (month_view);
 
-  priv->manager = manager;
+  priv->current_date = current_date;
+  gtk_widget_queue_draw (GTK_WIDGET (month_view));
 }
 
 /**
