@@ -63,66 +63,6 @@ month_item[12] =
 
 G_DEFINE_BOXED_TYPE (icaltimetype, icaltime, gcal_dup_icaltime, g_free)
 
-/* taken from eel/eel-gtk-extensions.c */
-static gboolean
-tree_view_button_press_callback (GtkWidget *tree_view,
-         GdkEventButton *event,
-         gpointer data)
-{
-  GtkTreePath *path;
-  GtkTreeViewColumn *column;
-
-  if (event->button == 1 && event->type == GDK_BUTTON_PRESS) {
-    if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view),
-             event->x, event->y,
-             &path,
-             &column,
-             NULL,
-             NULL)) {
-      gtk_tree_view_row_activated
-        (GTK_TREE_VIEW (tree_view), path, column);
-    }
-  }
-
-  return FALSE;
-}
-
-void
-gcal_gtk_tree_view_set_activate_on_single_click (GtkTreeView *tree_view,
-                                                 gboolean should_activate)
-{
-  guint button_press_id;
-
-  button_press_id = GPOINTER_TO_UINT
-    (g_object_get_data (G_OBJECT (tree_view),
-          "gd-tree-view-activate"));
-
-  if (button_press_id && !should_activate) {
-    g_signal_handler_disconnect (tree_view, button_press_id);
-    g_object_set_data (G_OBJECT (tree_view),
-         "gd-tree-view-activate",
-         NULL);
-  } else if (!button_press_id && should_activate) {
-    button_press_id = g_signal_connect
-      (tree_view,
-       "button_press_event",
-       G_CALLBACK  (tree_view_button_press_callback),
-       NULL);
-    g_object_set_data (G_OBJECT (tree_view),
-         "gd-tree-view-activate",
-         GUINT_TO_POINTER (button_press_id));
-  }
-}
-
-const gchar*
-gcal_get_group_name (const gchar *base_uri)
-{
-  if (g_strcmp0 (base_uri, "local:") == 0)
-    return "Local";
-  else
-    return "External";
-}
-
 icaltimetype*
 gcal_dup_icaltime (const icaltimetype *date)
 {
@@ -146,85 +86,6 @@ gcal_dup_icaltime (const icaltimetype *date)
   return new_date;
 }
 
-/*
- * gcal_get_source_name:
- *
- * This method assume it receive only the model inside GcalManager
- * */
-gchar*
-gcal_get_source_name (GtkTreeModel *model,
-                      const gchar  *uid)
-{
-  GtkTreeIter iter;
-  gboolean valid;
-  gchar *name;
-
-  name = NULL;
-  valid = gtk_tree_model_get_iter_first (model, &iter);
-  while (valid)
-    {
-      gchar *name_data;
-      gchar *uid_data;
-
-      gtk_tree_model_get (model, &iter,
-                          0, &uid_data,
-                          1, &name_data,
-                          -1);
-
-      if (g_strcmp0 (uid_data, uid) == 0)
-        {
-          name = g_strdup (name_data);
-
-          g_free (name_data);
-          g_free (uid_data);
-          break;
-        }
-
-      g_free (name_data);
-      g_free (uid_data);
-
-      valid = gtk_tree_model_iter_next (model, &iter);
-    }
-  return name;
-}
-
-gchar*
-gcal_get_source_uid (GtkTreeModel *model,
-                     const gchar  *name)
-{
-  GtkTreeIter iter;
-  gboolean valid;
-  gchar *uid;
-
-  uid = NULL;
-  valid = gtk_tree_model_get_iter_first (model, &iter);
-  while (valid)
-    {
-      gchar *name_data;
-      gchar *uid_data;
-
-      gtk_tree_model_get (model, &iter,
-                          0, &uid_data,
-                          1, &name_data,
-                          -1);
-
-      if (g_strcmp0 (name_data, name) == 0)
-        {
-          uid = g_strdup (uid_data);
-
-          g_free (name_data);
-          g_free (uid_data);
-          break;
-        }
-
-      g_free (name_data);
-      g_free (uid_data);
-
-      valid = gtk_tree_model_iter_next (model, &iter);
-    }
-  return uid;
-}
-
 gchar*
 gcal_get_weekday (gint i)
 {
@@ -273,6 +134,40 @@ gcal_get_pixbuf_from_color (GdkRGBA  *color,
   return pix;
 }
 
+GdkPixbuf*
+get_circle_pixbuf_from_color (GdkRGBA *color,
+                              gint     size)
+{
+  cairo_surface_t *surface;
+  cairo_t *cr;
+  GdkPixbuf *pix;
+
+  /* TODO: review size here, maybe not hardcoded */
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, size, size);
+  cr = cairo_create (surface);
+
+  cairo_set_source_rgba (cr,
+                         color->red,
+                         color->green,
+                         color->blue,
+                         color->alpha);
+  cairo_arc (cr, size / 2.0, size / 2.0, size / 2.0, 0., 2 * M_PI);
+  cairo_fill (cr);
+  cairo_destroy (cr);
+  pix = gdk_pixbuf_get_from_surface (surface,
+                                     0, 0,
+                                     size, size);
+  cairo_surface_destroy (surface);
+  return pix;
+}
+
+const gchar*
+get_color_name_from_source (ESource *source)
+{
+  ESourceSelectable *extension = E_SOURCE_SELECTABLE (e_source_get_extension (source, E_SOURCE_EXTENSION_CALENDAR));
+  return e_source_selectable_get_color (extension);
+}
+
 gint
 gcal_compare_event_widget_by_date (gconstpointer a,
                                    gconstpointer b)
@@ -300,22 +195,6 @@ gcal_compare_event_widget_by_date (gconstpointer a,
   return comparison;
 }
 
-void print_date (const gchar*        prefix,
-                 const icaltimetype* icaltime)
-{
-  gchar* temp = g_strdup_printf ("{y = %d, m = %d, d = %d, hour = %d:%d:%d, utc = %d, date = %d, daylight = %d, zone = %p}",
-                                 icaltime->year, icaltime->month,
-                                 icaltime->day, icaltime->hour,
-                                 icaltime->minute, icaltime->second,
-                                 icaltime->is_utc, icaltime->is_date,
-                                 icaltime->is_daylight, icaltime->zone);
-  g_debug ("%s: %s  --  zone: %s",
-           prefix,
-           temp,
-           icaltimezone_get_display_name ((icaltimezone*) icaltime->zone));
-  g_free (temp);
-}
-
 /**
  * get_desc_from_component:
  * @component:
@@ -398,15 +277,59 @@ get_uuid_from_component (ESource       *source,
   return uuid;
 }
 
+/**
+ * get_first_weekday:
+ *
+ * Copied from Clocks, which by itself is
+ * copied from GtkCalendar.
+ *
+ * Returns: the first weekday, from 0 to 6
+ */
 gint
 get_first_weekday (void)
 {
- const char *const s = nl_langinfo(_NL_TIME_FIRST_WEEKDAY);
+  int week_start;
+#ifdef HAVE__NL_TIME_FIRST_WEEKDAY
+  union { unsigned int word; char *string; } langinfo;
+  int week_1stday = 0;
+  int first_weekday = 1;
+  guint week_origin;
+#else
+  char *gtk_week_start;
+#endif
 
- if (s && *s >= 1 && *s <= 7)
-   return ((int)*s) - 1;
+#ifdef HAVE__NL_TIME_FIRST_WEEKDAY
+  langinfo.string = nl_langinfo (_NL_TIME_FIRST_WEEKDAY);
+  first_weekday = langinfo.string[0];
+  langinfo.string = nl_langinfo (_NL_TIME_WEEK_1STDAY);
+  week_origin = langinfo.word;
+  if (week_origin == 19971130) /* Sunday */
+    week_1stday = 0;
+  else if (week_origin == 19971201) /* Monday */
+    week_1stday = 1;
+  else
+    g_warning ("Unknown value of _NL_TIME_WEEK_1STDAY.\n");
 
- return 0;
+  week_start = (week_1stday + first_weekday - 1) % 7;
+#else
+  /* Use a define to hide the string from xgettext */
+# define GTK_WEEK_START "calendar:week_start:0"
+  gtk_week_start = dgettext ("gtk30", GTK_WEEK_START);
+
+  if (strncmp (gtk_week_start, "calendar:week_start:", 20) == 0)
+    week_start = *(gtk_week_start + 20) - '0';
+  else
+    week_start = -1;
+
+  if (week_start < 0 || week_start > 6)
+    {
+      g_warning ("Whoever translated calendar:week_start:0 for GTK+ "
+                 "did so wrongly.\n");
+      week_start = 0;
+    }
+#endif
+
+  return week_start;
 }
 
 /**
@@ -459,6 +382,33 @@ build_component_from_details (const gchar        *summary,
   return event;
 }
 
+/**
+ * icaltime_compare_date:
+ * @date1:
+ * @date2:
+ *
+ * Compare date parts of {@link icaltimetype} objects. Return negative value, 0 or positive value
+ * accordingly if date1 is before, same day of after date2.
+ * As a bonus it returns the amount of days passed between two days on the same year.
+ *
+ * Returns: negative, 0 or positive
+ **/
+gint
+icaltime_compare_date (const icaltimetype *date1,
+                       const icaltimetype *date2)
+{
+  if (date2 == NULL)
+    return 0;
+
+  if (date1->year < date2->year)
+    return -1;
+  else if (date1->year > date2->year)
+    return 1;
+  else
+    return time_day_of_year (date1->day, date1->month - 1, date1->year) -
+           time_day_of_year (date2->day, date2->month - 1, date2->year);
+}
+
 /* Function to do a last minute fixup of the AM/PM stuff if the locale
  * and gettext haven't done it right. Most English speaking countries
  * except the USA use the 24 hour clock (UK, Australia etc). However
