@@ -51,20 +51,8 @@ const gchar*  (*GcalTranslateFunc)                              (GtkWidget
 
 GType           icaltime_get_type                               (void)            G_GNUC_CONST;
 
-void            gcal_gtk_tree_view_set_activate_on_single_click (GtkTreeView           *tree_view,
-                                                                 gboolean               should_activate);
-
-
-const gchar*    gcal_get_group_name                             (const gchar           *base_uri);
-
 icaltimetype*   gcal_dup_icaltime                               (const icaltimetype    *date);
 
-gchar*          gcal_get_source_name                            (GtkTreeModel          *model,
-                                                                 const gchar           *uid);
-
-gchar*          gcal_get_source_uid                             (GtkTreeModel          *model,
-                                                                 const gchar           *name);
-
 gchar*          gcal_get_weekday                                (gint                   i);
 
 gchar*          gcal_get_month_name                             (gint                   i);
@@ -72,12 +60,14 @@ gchar*          gcal_get_month_name                             (gint
 GdkPixbuf*      gcal_get_pixbuf_from_color                      (GdkRGBA               *color,
                                                                  gint                   size);
 
+GdkPixbuf*      get_circle_pixbuf_from_color                    (GdkRGBA               *color,
+                                                                 gint                   size);
+
+const gchar*    get_color_name_from_source                      (ESource               *source);
+
 gint            gcal_compare_event_widget_by_date               (gconstpointer          a,
                                                                  gconstpointer          b);
 
-void            print_date                                      (const gchar*           prefix,
-                                                                 const icaltimetype*    icaltime);
-
 gchar*          get_desc_from_component                         (ECalComponent         *component,
                                                                  const gchar           *joint_char);
 
@@ -90,6 +80,9 @@ ECalComponent*  build_component_from_details                    (const gchar
                                                                  const icaltimetype    *initial_date,
                                                                  const icaltimetype    *final_date);
 
+gint            icaltime_compare_date                           (const icaltimetype    *date1,
+                                                                 const icaltimetype    *date2);
+
 /* code brought from evolution */
 gsize           e_strftime_fix_am_pm                            (gchar                 *str,
                                                                  gsize                  max,
