@@ -35,20 +35,25 @@
 #include <libecal/libecal.h>
 #include <libical/icaltime.h>
 
-struct _NewEventData
+typedef struct
 {
   gint               x;
   gint               y;
   icaltimetype      *start_date;
   icaltimetype      *end_date;
-};
+} NewEventData;
 
-typedef struct _NewEventData NewEventData;
+typedef struct
+{
+  GcalWindow *window;
+  gchar *uuid;
+} OpenEditDialogData;
 
 typedef struct
 {
   /* timeout ids */
   guint                save_geometry_timeout_id;
+  guint                notification_timeout;
 
   /* upper level widgets */
   GtkWidget           *main_box;
@@ -57,6 +62,8 @@ typedef struct
   GtkWidget           *search_bar;
   GtkWidget           *views_overlay;
   GtkWidget           *views_stack;
+  GtkWidget           *month_view;
+  GtkWidget           *year_view;
   GtkWidget           *notification;
   GtkWidget           *notification_label;
   GtkWidget           *notification_action_button;
@@ -89,6 +96,8 @@ typedef struct
   GcalWindowViewType   active_view;
   icaltimetype        *active_date;
 
+  icaltimetype        *current_date;
+
   /* states */
   gboolean             new_event_mode;
   gboolean             search_mode;
@@ -112,9 +121,9 @@ enum
 {
   PROP_0,
   PROP_ACTIVE_VIEW,
+  PROP_MANAGER,
   PROP_ACTIVE_DATE,
-  PROP_NEW_EVENT_MODE,
-  PROP_MANAGER
+  PROP_NEW_EVENT_MODE
 };
 
 #define SAVE_GEOMETRY_ID_TIMEOUT 100 /* ms */
@@ -163,8 +172,8 @@ static void           create_notification                (GcalWindow          *w
                                                           gchar               *message,
                                                           gchar               *button_label);
 
-static void           hide_notification                  (GtkWidget           *button,
-                                                          gpointer             user_data);
+static void           hide_notification                  (GcalWindow          *window,
+                                                          GtkWidget           *button);
 
 /* calendar management */
 static void           add_source                         (GcalManager         *manager,
@@ -260,12 +269,94 @@ key_pressed (GtkWidget *widget,
                                       event);
 }
 
+static void
+update_active_date (GcalWindow   *window,
+                    icaltimetype *new_date)
+{
+  GcalWindowPrivate *priv;
+
+  time_t range_start, range_end;
+  icaltimetype date;
+  icaltimetype *previous_date;
+  icaltimezone* default_zone;
+
+  priv = gcal_window_get_instance_private (window);
+  default_zone = gcal_manager_get_system_timezone (priv->manager);
+
+  previous_date = priv->active_date;
+  priv->active_date = new_date;
+  g_object_notify (G_OBJECT (window), "active-date");
+
+  /* year_view */
+  if (previous_date->year != new_date->year)
+    {
+      date = *new_date;
+      date.day = 1;
+      date.month = 1;
+      date.hour = 0;
+      date.minute = 0;
+      date.second = 0;
+      date.is_date = 0;
+      range_start = icaltime_as_timet_with_zone (date, default_zone);
+
+      date.day = 31;
+      date.month = 12;
+      date.hour = 23;
+      date.minute = 59;
+      range_end = icaltime_as_timet_with_zone (date, default_zone);
+
+      gcal_manager_set_subscriber (priv->manager, E_CAL_DATA_MODEL_SUBSCRIBER (priv->year_view), range_start, range_end);
+    }
+
+  /* month_view */
+  if (previous_date->month != new_date->month || previous_date->year != new_date->year)
+    {
+      date = *new_date;
+      date.day = 1;
+      date.hour = 0;
+      date.minute = 0;
+      date.second = 0;
+      date.is_date = 0;
+      range_start = icaltime_as_timet_with_zone (date, default_zone);
+
+      date.day = time_days_in_month (new_date->year, new_date->month - 1);
+      date.hour = 23;
+      date.minute = 59;
+      range_end = icaltime_as_timet_with_zone (date, default_zone);
+
+      gcal_manager_set_subscriber (priv->manager, E_CAL_DATA_MODEL_SUBSCRIBER (priv->month_view), range_start, range_end);
+    }
+
+  g_free (previous_date);
+}
+
+static gboolean
+update_current_date (GcalWindow *window)
+{
+  GcalWindowPrivate *priv = gcal_window_get_instance_private (window);
+  guint seconds;
+
+  if (priv->current_date == NULL)
+    priv->current_date = g_new0 (icaltimetype, 1);
+
+  *(priv->current_date) = icaltime_current_time_with_zone (gcal_manager_get_system_timezone (priv->manager));
+  *(priv->current_date) = icaltime_set_timezone (priv->current_date, gcal_manager_get_system_timezone (priv->manager));
+
+  gcal_month_view_set_current_date (GCAL_MONTH_VIEW (priv->month_view), priv->current_date);
+  gcal_year_view_set_current_date (GCAL_YEAR_VIEW (priv->year_view), priv->current_date);
+
+  seconds = 24 * 60 * 60 - (icaltime_as_timet (*(priv->current_date)) % (24 * 60 * 60));
+  g_timeout_add_seconds (seconds, (GSourceFunc) update_current_date, window);
+  return FALSE;
+}
+
 static void
 date_updated (GtkButton  *button,
               gpointer    user_data)
 {
   GcalWindowPrivate *priv;
 
+  icaltimetype *new_date;
   gboolean move_back, move_today;
 
   priv = gcal_window_get_instance_private (GCAL_WINDOW (user_data));
@@ -273,39 +364,38 @@ date_updated (GtkButton  *button,
   move_today = priv->today_button == (GtkWidget*) button;
   move_back = priv->back_button == (GtkWidget*) button;
 
+  new_date = gcal_dup_icaltime (priv->active_date);
+
   if (move_today)
     {
-      *(priv->active_date) =
-        icaltime_current_time_with_zone (
-            gcal_manager_get_system_timezone (priv->manager));
-      *(priv->active_date) =
-        icaltime_set_timezone (
-            priv->active_date,
-            gcal_manager_get_system_timezone (priv->manager));
+      *new_date = *(priv->current_date);
     }
   else
     {
       switch (priv->active_view)
         {
         case GCAL_WINDOW_VIEW_DAY:
-          priv->active_date->day += 1 * (move_back ? -1 : 1);
+          new_date->day += 1 * (move_back ? -1 : 1);
           break;
         case GCAL_WINDOW_VIEW_WEEK:
-          priv->active_date->day += 7 * (move_back ? -1 : 1);
+          new_date->day += 7 * (move_back ? -1 : 1);
           break;
         case GCAL_WINDOW_VIEW_MONTH:
-          priv->active_date->month += 1 * (move_back ? -1 : 1);
+          new_date->day = 1;
+          new_date->month += 1 * (move_back ? -1 : 1);
           break;
         case GCAL_WINDOW_VIEW_YEAR:
-          priv->active_date->year += 1 * (move_back ? -1 : 1);
+          new_date->year += 1 * (move_back ? -1 : 1);
           break;
         case GCAL_WINDOW_VIEW_LIST:
         case GCAL_WINDOW_VIEW_SEARCH:
           break;
         }
+
+      *new_date = icaltime_normalize (*new_date);
     }
-  *(priv->active_date) = icaltime_normalize (*(priv->active_date));
-  g_object_notify (user_data, "active-date");
+
+  update_active_date (user_data, new_date);
 }
 
 static void
@@ -616,16 +706,20 @@ create_notification (GcalWindow *window,
 }
 
 static void
-hide_notification (GtkWidget *button,
-                   gpointer   user_data)
+hide_notification (GcalWindow *window,
+                   GtkWidget  *button)
 {
-  GcalWindowPrivate *priv;
-
-  priv = gcal_window_get_instance_private (GCAL_WINDOW (user_data));
-  gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification),
-                                 FALSE);
+  GcalWindowPrivate *priv = gcal_window_get_instance_private (window);
+  gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification), FALSE);
+  priv->notification_timeout = 0;
 }
 
+static gboolean
+hide_notification_scheduled (gpointer window)
+{
+  hide_notification (GCAL_WINDOW (window), NULL);
+  return FALSE;
+}
 
 static void
 add_source (GcalManager *manager,
@@ -641,7 +735,6 @@ add_source (GcalManager *manager,
   GSimpleAction *action;
 
   gchar *item_name;
-  ESourceSelectable *extension;
 
   priv = gcal_window_get_instance_private (GCAL_WINDOW (user_data));
 
@@ -652,8 +745,7 @@ add_source (GcalManager *manager,
 
   /* retrieve the source's color & build item name */
   item_name = g_strdup_printf ("%s", e_source_get_uid (source));
-  extension = E_SOURCE_SELECTABLE (e_source_get_extension (source, E_SOURCE_EXTENSION_CALENDAR));
-  gdk_rgba_parse (&color, e_source_selectable_get_color (E_SOURCE_SELECTABLE (extension)));
+  gdk_rgba_parse (&color, get_color_name_from_source (source));
   pix = gcal_get_pixbuf_from_color (&color, 16);
 
   /* create the menu item */
@@ -936,22 +1028,26 @@ edit_dialog_closed (GtkDialog *dialog,
       break;
 
     case GCAL_RESPONSE_DELETE_EVENT:
-      /* delete the event */
-      create_notification (GCAL_WINDOW (user_data),
-                           _("Event deleted"),
-                           _("Undo"));
-      gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification),
-                                     TRUE);
-
-      /* FIXME: this will crash if the notification is still open */
       if (priv->event_to_delete != NULL)
-        g_free (priv->event_to_delete);
+        {
+          gcal_manager_remove_event (priv->manager, priv->event_to_delete->source, priv->event_to_delete->event_component);
+          g_clear_pointer (&(priv->event_to_delete), g_free);
+
+          create_notification (GCAL_WINDOW (user_data), _("Another event deleted"), _("Undo"));
+        }
+      else
+        {
+          create_notification (GCAL_WINDOW (user_data), _("Event deleted"), _("Undo"));
+        }
+
+      gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification), TRUE);
+      if (priv->notification_timeout != 0)
+        g_source_remove (priv->notification_timeout);
+      priv->notification_timeout = g_timeout_add_seconds (5, hide_notification_scheduled, user_data);
 
       priv->event_to_delete = g_new0 (GcalEventData, 1);
-      priv->event_to_delete->source =
-        gcal_edit_dialog_get_source (edit_dialog);
-      priv->event_to_delete->event_component =
-        gcal_edit_dialog_get_component (edit_dialog);
+      priv->event_to_delete->source = gcal_edit_dialog_get_source (edit_dialog);
+      priv->event_to_delete->event_component = gcal_edit_dialog_get_component (edit_dialog);
 
       uuid = gcal_edit_dialog_get_event_uuid (edit_dialog);
       /* hide widget of the event */
@@ -1025,9 +1121,9 @@ search_bar_revealer_toggled (GObject    *object,
 }
 
 static void
-remove_event (GtkWidget       *notification,
-                          GParamSpec      *spec,
-                          gpointer         user_data)
+remove_event (GtkWidget  *notification,
+              GParamSpec *spec,
+              gpointer    user_data)
 {
   GcalWindowPrivate *priv;
 
@@ -1062,16 +1158,39 @@ undo_remove_event (GtkButton *button,
       widgets = gcal_view_get_children_by_uuid (GCAL_VIEW (priv->views[priv->active_view]), uuid);
       g_list_foreach (widgets, (GFunc) gtk_widget_show, NULL);
 
+      g_object_unref (priv->event_to_delete->event_component);
       g_clear_pointer (&(priv->event_to_delete), g_free);
       g_list_free (widgets);
       g_free (uuid);
 
-      gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification),
-                                     FALSE);
+      g_source_remove (priv->notification_timeout);
+      hide_notification (GCAL_WINDOW (user_data), NULL);
     }
 
 }
 
+static gboolean
+schedule_open_edit_dialog_by_uuid (OpenEditDialogData *edit_dialog_data)
+{
+  GcalWindowPrivate *priv;
+  GList *widgets;
+
+  priv = gcal_window_get_instance_private (edit_dialog_data->window);
+  widgets = gcal_view_get_children_by_uuid (GCAL_VIEW (priv->month_view), edit_dialog_data->uuid);
+  if (widgets != NULL)
+    {
+      event_activated (NULL, widgets->data, edit_dialog_data->window);
+      g_list_free (widgets);
+      g_free (edit_dialog_data->uuid);
+      g_free (edit_dialog_data);
+      return FALSE;
+    }
+  else
+    {
+      return TRUE;
+    }
+}
+
 static void
 gcal_window_class_init(GcalWindowClass *klass)
 {
@@ -1099,6 +1218,15 @@ gcal_window_class_init(GcalWindowClass *klass)
                          GCAL_WINDOW_VIEW_MONTH,
                          G_PARAM_READWRITE));
 
+  g_object_class_install_property (
+      object_class,
+      PROP_MANAGER,
+      g_param_spec_pointer ("manager",
+                            "The manager object",
+                            "A weak reference to the app manager object",
+                            G_PARAM_CONSTRUCT_ONLY |
+                            G_PARAM_READWRITE));
+
   g_object_class_install_property (
       object_class,
       PROP_ACTIVE_DATE,
@@ -1118,15 +1246,6 @@ gcal_window_class_init(GcalWindowClass *klass)
                             FALSE,
                             G_PARAM_READWRITE));
 
-  g_object_class_install_property (
-      object_class,
-      PROP_MANAGER,
-      g_param_spec_pointer ("manager",
-                            "The manager object",
-                            "A weak reference to the app manager object",
-                            G_PARAM_CONSTRUCT_ONLY |
-                            G_PARAM_READWRITE));
-
   /* widgets */
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, header_bar);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, main_box);
@@ -1141,6 +1260,8 @@ gcal_window_class_init(GcalWindowClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, forward_button);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, views_overlay);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, views_stack);
+  gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, month_view);
+  gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, year_view);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, views_switcher);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, popover);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, search_view);
@@ -1168,7 +1289,10 @@ gcal_window_class_init(GcalWindowClass *klass)
 
   /* Event creation related */
   gtk_widget_class_bind_template_callback (widget_class, create_event);
+  gtk_widget_class_bind_template_callback (widget_class, create_event_detailed_cb);
+  gtk_widget_class_bind_template_callback (widget_class, show_new_event_widget);
   gtk_widget_class_bind_template_callback (widget_class, close_new_event_widget);
+  gtk_widget_class_bind_template_callback (widget_class, event_activated);
 
   /* Syncronization related */
   gtk_widget_class_bind_template_callback (widget_class, window_state_changed);
@@ -1180,6 +1304,10 @@ gcal_window_class_init(GcalWindowClass *klass)
 static void
 gcal_window_init (GcalWindow *self)
 {
+  GcalWindowPrivate *priv = gcal_window_get_instance_private (self);
+
+  priv->active_date = g_new0 (icaltimetype, 1);
+
   gtk_widget_init_template (GTK_WIDGET (self));
 }
 
@@ -1195,7 +1323,6 @@ gcal_window_constructed (GObject *object)
   GSettings *helper_settings;
   gchar *clock_format;
   gboolean use_24h_format;
-  gint i;
 
   if (G_OBJECT_CLASS (gcal_window_parent_class)->constructed != NULL)
     G_OBJECT_CLASS (gcal_window_parent_class)->constructed (object);
@@ -1246,42 +1373,32 @@ gcal_window_constructed (GObject *object)
   //gcal_week_view_set_use_24h_format (GCAL_WEEK_VIEW (priv->views[GCAL_WINDOW_VIEW_WEEK]), use_24h_format);
   //gtk_stack_add_titled (GTK_STACK (priv->views_stack), priv->views[GCAL_WINDOW_VIEW_WEEK], "week", _("Week"));
 
-  priv->views[GCAL_WINDOW_VIEW_MONTH] = gcal_month_view_new ();
-  gcal_month_view_set_manager (GCAL_MONTH_VIEW (priv->views[GCAL_WINDOW_VIEW_MONTH]), priv->manager);
+  priv->views[GCAL_WINDOW_VIEW_MONTH] = priv->month_view;
+  priv->views[GCAL_WINDOW_VIEW_YEAR] = priv->year_view;
+
   gcal_month_view_set_first_weekday (GCAL_MONTH_VIEW (priv->views[GCAL_WINDOW_VIEW_MONTH]), get_first_weekday ());
   gcal_month_view_set_use_24h_format (GCAL_MONTH_VIEW (priv->views[GCAL_WINDOW_VIEW_MONTH]), use_24h_format);
-  gtk_stack_add_titled (GTK_STACK (priv->views_stack), priv->views[GCAL_WINDOW_VIEW_MONTH], "month", _("Month"));
 
-  priv->views[GCAL_WINDOW_VIEW_YEAR] = g_object_new (GCAL_TYPE_YEAR_VIEW, NULL);
-  gcal_year_view_set_manager (GCAL_YEAR_VIEW (priv->views[GCAL_WINDOW_VIEW_YEAR]), priv->manager);
   gcal_year_view_set_first_weekday (GCAL_YEAR_VIEW (priv->views[GCAL_WINDOW_VIEW_YEAR]), get_first_weekday ());
   gcal_year_view_set_use_24h_format (GCAL_YEAR_VIEW (priv->views[GCAL_WINDOW_VIEW_YEAR]), use_24h_format);
-  gtk_stack_add_titled (GTK_STACK (priv->views_stack), priv->views[GCAL_WINDOW_VIEW_YEAR], "year", _("Year"));
 
   /* search view */
   gcal_search_view_connect (GCAL_SEARCH_VIEW (priv->search_view), priv->manager);
   gcal_search_view_set_time_format (GCAL_SEARCH_VIEW (priv->search_view), use_24h_format);
 
-  /* current date hook */
-  gcal_year_view_set_current_date (GCAL_YEAR_VIEW (priv->views[GCAL_WINDOW_VIEW_YEAR]), NULL);
-
-  /* signals connection/handling */
-  /* only GcalView implementations */
-  for (i = 0; i < 4; ++i)
-    {
-      if (priv->views[i] != NULL)
-        {
-          g_object_bind_property (GCAL_WINDOW (object), "active-date", priv->views[i], "active-date",
-                                  G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
-
-          g_signal_connect (priv->views[i], "create-event", G_CALLBACK (show_new_event_widget), object);
-          g_signal_connect (priv->views[i], "create-event-detailed", G_CALLBACK (create_event_detailed_cb), object);
-          g_signal_connect (priv->views[i], "event-activated", G_CALLBACK (event_activated), object);
-        }
-    }
-
   /* refresh timeout, first is fast */
   priv->refresh_timeout_id = g_timeout_add (FAST_REFRESH_TIMEOUT, (GSourceFunc) refresh_sources, object);
+
+  /* calendars popover */
+  if (gcal_manager_load_completed (priv->manager))
+    {
+      GList *sources, *l;
+      sources = gcal_manager_get_sources_connected (priv->manager);
+      for (l = sources; l != NULL; l = g_list_next (l))
+        add_source (priv->manager, l->data, gcal_manager_source_enabled (priv->manager, l->data), object);
+
+      g_list_free (sources);
+    }
 }
 
 static void
@@ -1291,8 +1408,8 @@ gcal_window_finalize (GObject *object)
 
   priv = gcal_window_get_instance_private (GCAL_WINDOW (object));
 
-  if (priv->active_date != NULL)
-    g_free (priv->active_date);
+  g_free (priv->active_date);
+  g_free (priv->current_date);
 
   if (priv->views_switcher != NULL)
     g_object_unref (priv->views_switcher);
@@ -1319,19 +1436,18 @@ gcal_window_set_property (GObject      *object,
                                    priv->views[priv->active_view]);
       return;
     case PROP_ACTIVE_DATE:
-      if (priv->active_date != NULL)
-        g_free (priv->active_date);
-      priv->active_date = g_value_dup_boxed (value);
+      update_active_date (GCAL_WINDOW (object), g_value_dup_boxed (value));
       return;
     case PROP_NEW_EVENT_MODE:
       set_new_event_mode (GCAL_WINDOW (object), g_value_get_boolean (value));
       return;
     case PROP_MANAGER:
       priv->manager = g_value_get_pointer (value);
-      g_signal_connect (priv->manager, "source-added",
-                        G_CALLBACK (add_source), object);
-      g_signal_connect (priv->manager, "source-removed",
-                        G_CALLBACK (remove_source), object);
+      g_signal_connect (priv->manager, "source-added", G_CALLBACK (add_source), object);
+      g_signal_connect (priv->manager, "source-removed", G_CALLBACK (remove_source), object);
+
+      gcal_year_view_set_manager (GCAL_YEAR_VIEW (priv->year_view), priv->manager);
+      update_current_date (GCAL_WINDOW (object));
       return;
     }
 
@@ -1427,7 +1543,7 @@ gcal_window_new_with_view_and_date (GcalApplication   *app,
 
   manager = gcal_application_get_manager (GCAL_APPLICATION (app));
 
-  win  =  g_object_new (GCAL_TYPE_WINDOW, "application", GTK_APPLICATION (app), "active-date", date, "manager", manager,
+  win  =  g_object_new (GCAL_TYPE_WINDOW, "application", GTK_APPLICATION (app), "manager", manager, "active-date", date,
                         NULL);
 
   /* loading size */
@@ -1491,3 +1607,37 @@ gcal_window_set_search_mode (GcalWindow *window,
   gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (priv->search_bar),
                                   enabled);
 }
+
+void
+gcal_window_set_search_query (GcalWindow  *window,
+                              const gchar *query)
+{
+  GcalWindowPrivate *priv = gcal_window_get_instance_private (window);
+  gtk_entry_set_text (GTK_ENTRY (priv->search_entry), query);
+}
+
+void
+gcal_window_open_event_by_uuid (GcalWindow  *window,
+                                const gchar *uuid)
+{
+  GcalWindowPrivate *priv;
+  GList *widgets;
+
+  priv = gcal_window_get_instance_private (window);
+
+  /* XXX: show events on month view */
+  gtk_stack_set_visible_child (GTK_STACK (priv->views_stack), priv->month_view);
+  widgets = gcal_view_get_children_by_uuid (GCAL_VIEW (priv->month_view), uuid);
+  if (widgets != NULL)
+    {
+      event_activated (NULL, widgets->data, window);
+      g_list_free (widgets);
+    }
+  else
+    {
+      OpenEditDialogData *edit_dialog_data = g_new0 (OpenEditDialogData, 1);
+      edit_dialog_data->window = window;
+      edit_dialog_data->uuid = g_strdup (uuid);
+      g_timeout_add_seconds (2, (GSourceFunc) schedule_open_edit_dialog_by_uuid, edit_dialog_data);
+    }
+}
